﻿Imports System.Threading
Imports System.Text


Public Class frmMain
    Public chanCmd As Byte = 0
    Public chanBank1 As Byte = 0
    Public chanBank2 As Byte = 0


    Delegate Sub SetTextCallback(ByVal [text] As String)

    Public Function SetBit(ByVal InByte As Byte, ByVal Bit As Byte) As Byte
        Return InByte Or (2 ^ Bit)
    End Function

    Public Function ClearBit(ByVal InByte As Byte, ByVal Bit As Byte) As Byte
        Return InByte And Not (2 ^ Bit)
    End Function

    Public Function IsBitSet(ByVal InByte As Byte, ByVal Bit As Byte) As Boolean
        Return ((InByte And (2 ^ Bit)) > 0)
    End Function

    Private Sub SetText(ByVal [text] As String)
        ' InvokeRequired required compares the thread ID of the
        ' calling thread to the thread ID of the creating thread.
        ' If these threads are different, it returns true.
        If Me.listInfo.InvokeRequired Then
            Dim d As New SetTextCallback(AddressOf SetText)
            Me.Invoke(d, New Object() {[text]})
        Else
            Me.listInfo.Items.Add([text])
        End If
    End Sub

    Private Sub Serial_DataReceived(ByVal sender As System.Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) Handles Serial.DataReceived
        SetText(Serial.ReadExisting)
    End Sub

    Private Sub btnSendCmd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSendCmd.Click
        ' The Gazatta16 firmware looks for a command in this format:
        ' Bytes 1-4 Board ID
        ' Byte 5 Command Code
        ' Byte 6 Channel state for channels 1 - 8
        ' Byte 7 Channel state for channels 9 - 16

        Serial.WriteLine(txtBoardID.Text & Chr(chanCmd) & Chr(chanBank1) & Chr(chanBank2))
        listInfo.Items.Add("Sent: " & CStr(chanBank1) & "(" & Chr(chanBank1) & ") and " & CStr(chanBank2) & "(" & Chr(chanBank1) & ")")
    End Sub

    Private Sub btnOpenPort_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOpenPort.Click
        btnOpenPort.Enabled = False
        btnClosePort.Enabled = True
        btnSendCmd.Enabled = True
        btnChangeBoardID.Enabled = True
        btnShowBoardID.Enabled = True
        Serial.BaudRate = 250000 ' 115200 is used for slower systems or with serial controls (MSCOMM32.OCX for instance) that don't support
        '                          the higher 250kbit/sec rate.
        Serial.PortName = "COM5"
        Serial.Encoding = Encoding.Default  ' This is very important!  Without it, the .Net CLR will feed out 7 bit ASCII which causes issues.

        Serial.Open()
    End Sub

    Private Sub btnClosePort_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClosePort.Click
        Serial.Close()
        btnOpenPort.Enabled = True
        btnClosePort.Enabled = False
        btnSendCmd.Enabled = False
        btnChangeBoardID.Enabled = False
        btnShowBoardID.Enabled = False
    End Sub

    Private Sub frmMain_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        txtBoardID.Text = "GAZ1"
        btnSendCmd.Enabled = False
        btnChangeBoardID.Enabled = False
        btnShowBoardID.Enabled = False
        lblChan1.Text = ""
        lblChan2.Text = ""

    End Sub

    Private Sub chkChan1_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkChan1.CheckedChanged,
        chkChan2.CheckedChanged, chkChan3.CheckedChanged, chkChan4.CheckedChanged, chkChan5.CheckedChanged, chkChan6.CheckedChanged,
        chkChan7.CheckedChanged, chkChan8.CheckedChanged, chkChan9.CheckedChanged, chkChan10.CheckedChanged, chkChan11.CheckedChanged,
        chkChan12.CheckedChanged, chkChan13.CheckedChanged, chkChan14.CheckedChanged, chkChan15.CheckedChanged, chkChan15.CheckedChanged,
        chkChan16.CheckedChanged
        Dim index As Byte = CByte(sender.tag)
        Dim checked As Boolean = sender.checked

        If index <= 8 Then ' handles channels 1..8
            If checked Then chanBank1 = SetBit(chanBank1, index - 1) Else chanBank1 = ClearBit(chanBank1, index - 1)
            lblChan1.Text = CStr(chanBank1)
        Else
            index = index - 8 ' handles channels 9..16
            If checked Then chanBank2 = SetBit(chanBank2, index - 1) Else chanBank2 = ClearBit(chanBank2, index - 1)
            lblChan2.Text = CStr(chanBank2)
        End If
    End Sub

    Private Sub radOff_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles radOff.CheckedChanged
        If radOff.Checked Then
            chanCmd = 2
        End If
    End Sub

    Private Sub radOn_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles radOn.CheckedChanged
        If radOn.Checked Then
            chanCmd = 1
        End If
    End Sub

    Private Sub radAllOn_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles radAllOn.CheckedChanged
        If radAllOn.Checked Then
            chanCmd = 3
        End If
    End Sub

    Private Sub radAllOff_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles radAllOff.CheckedChanged
        If radAllOff.Checked Then
            chanCmd = 4
        End If
    End Sub

    Private Sub radFollowState_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles radFollowState.CheckedChanged
        If radFollowState.Checked Then
            chanCmd = 0
        End If
    End Sub

    Private Sub btnChangeBoardID_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnChangeBoardID.Click
        Dim newId As String = InputBox("Enter new board ID, 4 characters maximum.")
        newId = Microsoft.VisualBasic.UCase(Microsoft.VisualBasic.Left(newId, 4))
        If MsgBox("Are you SURE you wish to reset the board " & txtBoardID.Text & " ID to " & newId & "?", MsgBoxStyle.YesNo, "Warning!") = MsgBoxResult.Yes Then
            If MsgBox("Please make sure you have only a single board connected to the currently open port and then click OK to change the board ID.", MsgBoxStyle.YesNo, "Ready!") = MsgBoxResult.Yes Then
                chanCmd = 5
                Dim out_value As String = txtBoardID.Text + Chr(chanCmd) + newId + Chr(13)
                Serial.WriteLine(out_value)
                txtBoardID.Text = newId
            End If
        End If
    End Sub

    Private Sub btnShowBoardID_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnShowBoardID.Click
        chanCmd = 6
        Dim out_value As String = txtBoardID.Text + Chr(chanCmd) + Chr(13)
        Serial.WriteLine("I" + Chr(13))
    End Sub
End Class
